#!/usr/bin/perl -w
#
# by adam mathes
# 
# originally written ~ Nov 5, 2003
# updated for distribution ~ Feb 25, 2004
#
# for license information see licences.txt
#
# for documentation see readme.txt
#  

use Email::Folder;
use Email::Simple;
use HTML::Template;
use HTML::FromText;
use POSIX;
use Config::Natural;
use vars qw( $t2h $t2c $config $OUTPUT_DIR $OUTFILE $MESSAGE_TEMPLATE $ARCHIVE_SIZE
		$RSS_TEMPLATE $RSSFILE $CDF_TEMPLATE $CDFFILE $PENDING_MBOX $APPROVED_MBOX );
use strict;

$config = new Config::Natural 'dripmail.conf';

$ARCHIVE_SIZE = $config->param('ARCHIVE_SIZE');
$OUTPUT_DIR = $config->param('OUTPUT_DIR');
$OUTFILE = "$OUTPUT_DIR/" . $config->param('OUTFILE');
$MESSAGE_TEMPLATE = $config->param('MESSAGE_TEMPLATE');
$RSS_TEMPLATE = $config->param('RSS_TEMPLATE');
$RSSFILE = "$OUTPUT_DIR/" . $config->param('RSSFILE');
$CDF_TEMPLATE = $config->param('CDF_TEMPLATE');
$CDFFILE = "$OUTPUT_DIR/" . $config->param('CDFFILE');
$APPROVED_MBOX = $config->param('APPROVED_MBOX');
$PENDING_MBOX = $config->param('PENDING_MBOX');


$t2h = HTML::FromText->new({
	paras	=> 1,
	urls 	=> 1,
	email	=> 0,
	lines   => 1
});

$t2c = HTML::FromText->new({
        paras   => 0,
        urls    => 1,
        email   => 0,
        lines   => 0
});

my $mail_file = $APPROVED_MBOX;

my $folder = Email::Folder->new($mail_file);
my $messages = GetMessages($folder);
my $num_messages = @$messages;

# front page
my @recent = reverse @$messages;
@recent = splice @recent, 0, $ARCHIVE_SIZE;
my %s;
$s{messages} = \@recent;
$s{archives} = GetArchiveLinks($num_messages);
$s{pending} = GetPending(); 
my $output = ParseTemplate($MESSAGE_TEMPLATE, %s);
WriteToFile($output, $OUTFILE);

# rss feed, cdf feed
$output = ParseTemplate($RSS_TEMPLATE, %s);
WriteToFile($output, $RSSFILE);
$output = ParseTemplate($CDF_TEMPLATE, %s);
WriteToFile($output, $CDFFILE);

# archives
my $j;
$j=1;
while(@$messages) {
	my @segment = splice @$messages, 0, $ARCHIVE_SIZE;
	$s{messages} = \@segment;
	$s{pageNum} = $j;
	$output = ParseTemplate($MESSAGE_TEMPLATE, %s);
	WriteToFile($output, "$OUTPUT_DIR/$j.html");
	$j++; 
}


sub GetArchiveLinks {
	my $x = shift;
	my @links;
	
	my $pages = ceil($x / $ARCHIVE_SIZE);
	for(my $i=1; $i<=$pages; $i++) {
		my %l;
		$l{id} = "$i";
		push @links, \%l;
	}    
#	@links = reverse @links;
	return \@links;
}



sub GetMessages {
	my $folder = shift;
	my @simplemails = $folder->messages;
	my @messages;
	my $i=1;
	foreach my $mail (@simplemails) { 
		my %m;
		$m{date} = ToClean(join ' ', $mail->header('Date'));
		$m{from} = ToClean(join ' ', $mail->header('From'));
		$m{to} = ToClean(join ' ', $mail->header('To'));
		$m{subject} = ToClean(join ' ', $mail->header('Subject'));
		$m{body} = ToHTML(MangleEmails($mail->body));
		$m{id} = $i;
		$m{pagenum} = ceil ($i / $ARCHIVE_SIZE);
		$m{shortbody} = ShortBody(ToClean($mail->body));
		$i++;
		push @messages, \%m;
	}
	return \@messages;
}


sub ShortBody {
	my $s = shift;
	my @s = split ' ', $s;
	@s = splice @s, 0, 10;
	$s = join ' ' , @s; 
}


sub SortMessages {
  my $messages = shift;
  my $sort_field = shift;

  my @x = sort { $$main::a{$sort_field} cmp $$main::b{$sort_field}}  @$messages;
  return \@x;
}



sub ToHTML {
	my $s = shift;
	my $html = $t2h->parse( $s );
	return $html;
}

sub ToClean {
	my $s = shift;

#	$s =~ s/(\w[-.\w]+\@[-.\w]+\.\w{2,3})\W//gis;
#	$s =~ s/[-.\w]+\@[-.\w]+.\w{2,3}\W//gis;
#	$s =~ s/<//gis;

	$s = MangleEmails($s);
	my $html = $t2c->parse( $s );
	return $html;
}

sub ParseTemplate {
  (my $filename, my %s) = @_;

  my $template = HTML::Template->new(die_on_bad_params=>0,
                                     filename => "$filename",
                                     loop_context_vars => 1,
                                     global_vars=>1);
  $template->param(%s);
  my $output = $template->output;
  return $output;
}

sub WriteToFile {
  my $output = shift;
  my $filename = shift;

  open(OUTPUT,">$filename") || die "couldn't open $filename";
  print OUTPUT $output;
  close(OUTPUT);
  chmod(0775, $filename);
}


sub GetPending {
	my $folder = Email::Folder->new($PENDING_MBOX);
        my @simplemails = $folder->messages;
	my $num_messages = @simplemails;
	return $num_messages;
}

sub MangleEmails {
	my $s = shift;
	$s =~ s/(.*)\@(.*)/$1\@[xxx]/g;
	return $s;
}
